package main;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.image.BufferedImage;
import java.util.ArrayList;

import javax.swing.JFrame;

import charcter.Drawable;
import charcter.EatJudgment;
import charcter.kuma.Kuma;
import charcter.message.KumaEatMessageCreater;
import charcter.message.Message;
import charcter.message.MessageCreater;
import charcter.shika.RandShikaCreater;
import charcter.shika.Shika;
import charcter.shika.ShikaCreater;

public class Game {

	private static final int WIDTH = 640;
	private static final int HEIGHT = 800;
	
	private static final int SHIKA_INT = 10;
	
	/*食べ残した場合に狭くなるピクセル*/
	private static final int PENALTY = 50;
	
	public static final int RIGHT = 1;
	public static final int LEFT = 2;
		
	/*-------------------------------*/
	JFrame gameWindow;
	GamePanel gamePanel;
	
	/*-------------------------------*/
	BufferedImage drawImage;
	
	
	/*-------------------------------*/
	int keyState = 0;	
	int shikaInterval = 0;
			
	/*-------------------------------*/
	
	GameThread gameThread; 
	
	Ending ending;
	
	/*-------------------------------*/
	ArrayList<Drawable> drawableObject;
	
	Kuma kuma;
	ArrayList<Shika> shikaList;
	ArrayList<Message> messageList;
	
	
	ShikaCreater shikaCreater;
	MessageCreater messageCreater;

	Score score;
	
	EatJudgment judge;
	
	
	
	public Game(){
		init();
	}
	
	
	private void init(){

		gamePanel = new GamePanel(this);
//		gamePanel.setPreferredSize(new Dimension(WIDTH,HEIGHT));
//		gamePanel.setSize(new Dimension(WIDTH,HEIGHT));
		
		
		gameWindow = new JFrame();
		gameWindow.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		gameWindow.setLocation(100,100);		
		gameWindow.setResizable(false);
		gameWindow.getContentPane().add(gamePanel);
		
		gameWindow.setTitle("地デジの波には負けないクマー");
		
		gameWindow.addKeyListener(new KeyAdapter(){
			
			@Override
			public void keyPressed(KeyEvent e){
				int keyCode = e.getKeyCode();				
				if(keyCode == KeyEvent.VK_RIGHT)
					setKeyState(Game.RIGHT);
				else if(keyCode == KeyEvent.VK_LEFT){
					setKeyState(Game.LEFT);
				}
			}
			
		});
		gameWindow.setSize(new Dimension(WIDTH,HEIGHT));
//		gameWindow.pack();
		
		
		drawableObject = new ArrayList<Drawable>();
	
		messageCreater = new KumaEatMessageCreater();
		messageList = new ArrayList<Message>();
		
		shikaCreater = new RandShikaCreater(gamePanel);
		shikaList = new ArrayList<Shika>();
		
		kuma = new Kuma(0,gamePanel,true);
		drawableObject.add(kuma);
				
		score = new Score();
		drawableObject.add(score);
		
		judge = new EatJudgment(kuma);
		
		gameThread = new GameThread();
		gameWindow.setVisible(true);
	}
	
	public void startGame(){
		if(!gameThread.isRunning())gameThread.start();		
	}
	
	public void endGame(){
		gameThread.dispose();
		ending = new Ending("あんたは"+Integer.toString(score.getScore())+"クマー");
		gameWindow.getContentPane().remove(gamePanel);
		gameWindow.getContentPane().add(ending);		
		gameWindow.setSize(ending.getWidth(),ending.getHeight());
		gameWindow.setLocationRelativeTo(null);
	}
	
	public void drawImage(Graphics g){
		Graphics2D g2 = (Graphics2D)g;
		
		for(Drawable d : drawableObject)d.draw(g2);
		for(Shika s : shikaList)s.draw(g2);
		for(Message m : messageList)m.draw(g2);		
	}
		
	public synchronized void action(){
		
		if(shikaInterval++ > SHIKA_INT){
			shikaInterval = 0;
			shikaList.add(shikaCreater.createShika());
		}
		
		/*shika*/
		ArrayList<Shika> delShikaList = new ArrayList<Shika>(); 
		for(Shika s : shikaList){			
			if(s.getY() > gamePanel.getHeight()){
				if(Kuma.getHeight() > gamePanel.getHeight())endGame();
				gamePanel.setSize(gamePanel.getWidth(),gamePanel.getHeight()-PENALTY);
				gameWindow.setLocation(gameWindow.getLocation().x,gameWindow.getLocation().y+PENALTY);
				gameWindow.setSize(gameWindow.getWidth(),gameWindow.getHeight()-PENALTY);				
				delShikaList.add(s);
				messageList.add(messageCreater.createMessage((s.getX()+Shika.getWidth())/2,s.getY()+Shika.getHeight(),gamePanel.getSize()));				
			}else if( judge.eatJudge(s) ){
				score.addScore(1);
				delShikaList.add(s);
				messageList.add(messageCreater.createMessage((s.getX()+Shika.getWidth())/2,s.getY()+Shika.getHeight(),gamePanel.getSize()));
			}
			else s.action();
		}
		for(Shika s : delShikaList)shikaList.remove(s);
		delShikaList.clear();
		
		/*message*/
		ArrayList<Message> delMessageList = new ArrayList<Message>();
		for(Message m : messageList){
			if(m.isDispose())delMessageList.add(m);
			else m.move();
		}
		for(Message m : delMessageList)messageList.remove(m);
		delMessageList.clear();
		
		
		if(keyState == RIGHT){
			kuma.toRight();
			
		}else if(keyState == LEFT){
			kuma.toLeft();
		}
		kuma.move();
	}
	
	public synchronized void setKeyState(int s){
		keyState = s;
	}
	
	/**ゲームスレッド*/
	class GameThread implements Runnable{
		
		boolean runFlag;
		
		GameThread(){
			runFlag = false;
		}
		
		public void start(){
			runFlag = true;
			new Thread(this).start();
		}
		
		public synchronized void dispose(){
			runFlag = false;
		}
		
		public synchronized final boolean isRunning(){
			return runFlag;
		}
		
		@Override
		public void run(){
			while(runFlag){
				action();
				try {
					Thread.sleep(50);
				} catch (InterruptedException e) {					
					e.printStackTrace();
				}
			}
		}
	}	
}
